/**
 * A class for a Mancala board_. The following board_ layout is assumed
 *
 * Holes num:
 *
 *     13  12  11  10  9   8
 *  0                          7
 *     1   2   3   4   5   6   
 *
 *
 * Corresponding store num:
 *
 * 2                           1
 *
 * Corresponding house num:
 *
 *    12  11  10  9   8   7
 *
 *    1   2   3   4   5   6
 *      
 *
 *
 * Any even number of houses is allowed. 
 *
 */
#include <iostream>
#include <sstream>
#include <stdexcept>
#include <vector>
#include <cassert>

#include "Board.h"

using namespace std;

//const int INITIAL_NUMBER_OF_SEEDS_PER_HOUSE = 3;


/** Constructor.
 *
 * @param holes is the number of holes, including both house and store, in
 * the board_. Must be an even number.
 * @param seeds is the initial number of seeds per hole.
 *
 * @exception std::invalid_argument.
 */

Board::Board() {}

    
Board::Board(int houses, int seeds)
{
    int holes = houses + 2;

    if (holes % 2 != 0 || holes <= 0)
    {
        ostringstream msg;
        msg << "'holes' = " << holes << ": "
            << "The number of holes in a board_ must be equal";
        throw invalid_argument(msg.str());
    }

    store2_ = 0;
    store1_ = holes / 2;

    board_.reserve(holes);
    for (int i = 0; i < holes; ++i)
    {
        if (i == store1_ || i == store2_)
            board_.push_back(0);
        else
            board_.push_back(seeds);
    }
}


int & Board::operator[] (int hole)
{
    if (hole < 0 || hole > static_cast<int>(board_.size()-1))
        throw invalid_argument("argument out of range");
    return board_[hole];
}


#if 0
/* house number starts at 1 */
bool Board::sow(int house, Half playersHalf, int * endId, Board::HoleType * endType)
{
    int hole;

    assert (playersHalf == SOUTH || playersHalf == NORTH);

    try
    {
        hole = house2hole(house);        
    }
    catch (...)
    {
        return false;   
    }

    if (getHalf(hole) != playersHalf)
        return false;

    int seeds = getSeedsInHouse(house);
    
    if (seeds == 0)
        return false;

    board_[hole] = 0;

    int opponentStore = getStoreHole((playersHalf == SOUTH) ? NORTH : SOUTH);
    while (seeds)
    {
        hole = ( hole == static_cast<int>( board_.size())-1 ) ? 0 : hole + 1;
        if (hole != opponentStore)
        {
            ++board_[hole];
            --seeds;
        }
    }

    *endType = (hole == store1_ || hole == store2_) ? STORE : HOUSE;
    *endId = *endType == STORE ? hole2store(hole) : hole2house(hole);

    return true;
}
#endif


bool Board::isHalfEmpty(Half half) const
{
    int isEmpty = true;
    int hole = half == SOUTH ? store2_ + 1 : store1_ + 1;
    int end = half == SOUTH ? store1_ : store2_;

    while (hole != end)
    {
        if (board_[hole] != 0)
        {
            isEmpty = false;
            break;
        }
        hole = ( hole == static_cast<int>( board_.size())-1 ) ? 0 : hole + 1;            
    }
    
    return isEmpty;
}



int Board::getSeedsInHouse(int house) const
{
    return board_[house2hole(house)];
}


int Board::getSeedsInStore(int store) const
{
    return board_[store2hole(store)];
}


Board::Half Board::getHalf (int hole) const
{
    if (hole <= store1_ && hole != store2_) 
        return SOUTH;
    else
        return NORTH;
}


int Board::getOppositeHole( int hole ) const
{
    int opposite;

    checkRange (hole);

    if ( hole == 0 )
        opposite = board_.size() / 2;
    else if ( hole == static_cast<int>(board_.size()) / 2 )
        opposite = 0;
    else
        opposite = board_.size() - hole;

    return opposite;
}


int Board::hole2house(int hole) const
{
    checkRange (hole);
    return hole < store1_ ? hole : hole - 1;
}

/**
 * Returns the store number which HOLE corresponds to. Returns 0 if hole does
 * not correspond to any store.
 */
int Board::hole2store(int hole) const
{
    checkRange (hole);
    if ( hole == store1_ )
        return 1;
    else if ( hole == store2_)
        return 2;
    else
        return 0;
}

int Board::house2hole(int house) const
{
    if (house < 1 || house > static_cast<int>(board_.size()-2))
        throw invalid_argument("Invalid house number");
    return house < store1_ ? house : house + 1;
}

int Board::store2hole(int store) const
{
    if (store < 1 || store > 2)
        throw invalid_argument("Invalid store number");
    return store == 1 ? store1_ : store2_;

}

#if 0
// num is number from 1...
int Board::getHole(int num, HoleType type) const
{
    int hole;
    if (type == HOUSE)
    {
        if (num < 1 || num > board_.size() - 2)
            throw invalid_argument("Invalid house number");
        hole = (num < store1_) ? num : num + 1;
    }
    else if (type == STORE)
    {
    }
    else
        throw invalid_argument("Unknown hole type");
    return hole;
}
#endif


int Board::getStoreHole(Half half) const
{
    int hole;
    if (half == SOUTH)
        hole = store1_;
    else if (half == NORTH)
        hole = store2_;
    else
        throw invalid_argument("Only store for half SOUTH and NORTH");
    return hole;
}

size_t Board::size () const
{
    return board_.size();
}


bool Board::checkRange( int hole ) const
{
    if ( hole < 0 || hole > static_cast<int>(board_.size()-1) )
        throw invalid_argument("Invalid hole number");
    return true;
}

