
#include <cassert>
#include <string>

#include "Game.h"
#include "Board.h"
#include "Player.h"
#include "IEngine.h"


Game::Game ( IEngine * engine, int houses, int seeds )
    : engine_(engine),
      board_( houses, seeds ),
      finished_(false)
{ }


void Game::player1 ( std::string name, bool user )
{
    player1_ = new Player(name, user ? Player::USER : Player::COMPUTER, &board_, Board::SOUTH);
    activePlayer_ = player1_;
}

void Game::player2 ( std::string name, bool user )
{
    player2_ = new Player(name, user ? Player::USER : Player::COMPUTER, &board_, Board::NORTH);
}


/**
 * Makes the active user make a move (seed) on HOUSE and updates game state.
 * Returns TRUE if the move is valid.
 */
bool Game::makeMove( int house )
{
    assert( player1_ );
    assert( player2_ );

    if (finished_)
        return false;

    IEngine::MoveResult res = engine_->makeMove( activePlayer_, &board_, house );
    switch (res)
    {
        case IEngine::INVALID_MOVE:
            return false;
        case IEngine::VALID_MOVE_EXTRA_TURN:
            return true;
        case IEngine::VALID_MOVE_CHANGE_TURN:
            activePlayer_ = activePlayer_ == player1_ ? player2_ : player1_;        
            return true;
        case IEngine::VALID_MOVE_GAME_FINISHED:
            finished_ = true;
            return true;
        default:
            std::cerr << __FUNCTION__ << ": Got invalid move result" << std::endl;
            assert(true);
    }

    return false;
}

/**
 * Returns the name of the winner or an empty string if the game ends in a
 * draw. If the game is not finished a 0 pointer is returned.
 */
std::string Game::getWinner () const
{
    if (finished_)
    {
        Player * winner = engine_->getWinner (player1_, player2_, &board_);
        if ( winner == 0 )
            return "";
        else
            return winner->name();
    }
    return 0;
}

bool Game::isFinished  () const
{
    return finished_;
}

int Game::getSeedsInHouse ( int house ) const
{
    return board_.getSeedsInHouse( house );
}


int Game::getSeedsInStorage ( int storage ) const
{
    return board_.getSeedsInStore ( storage );
}

std::string Game::getActivePlayerName () const
{
    return activePlayer_->name();
}

std::string Game::getPlayer1Name () const
{
    return player1_->name();
}

std::string Game::getPlayer2Name () const
{
    return player2_->name();
}

